import numpy as np

import tools as T

# %%
__all__ = [
'equal',
'merge_dict',
]

# %%
def equal(array):
    if len(array)<=1:
        return True
    else:
        return (array[0]==array[1:]).all()

def merge_dict(ld):
    assert type(ld) == list, f'must give list of dicts, received: {type(ld)}'
    assert T.equal([list(d.keys()) for d in ld]), 'keys for every dict in the list of dicts must be the same'
    keys = ld[0].keys()
    merged_d = {}
    for k in keys:
        lv = [d[k] for d in ld] # list of values
        try:
            merged_d[k] = np.concatenate(lv, axis=0)
        except ValueError:
            merged_d[k] = np.array(lv)
    return merged_d
    
# %%
if __name__ == '__main__':
    ld = [{i:i+1 for i in range(5)} for j in range(3)]
    merge_dict(ld)
    ld = [{i:np.arange(i+1) for i in range(5)} for j in range(3)]
    merge_dict(ld)



import torch

def get_device(model):
    return next(model.parameters()).device

def spread_device(job_num = None):
    if job_num != None:
        gpu_id = job_num % torch.cuda.device_count()
    else:
        gpu_id = torch.cuda.default_stream().device.index
    device = torch.device(f"cuda:{gpu_id}" if torch.cuda.is_available() else "cpu")
    return device

def multiprocessing_device(gpu_id = None):
    '''
    if gpu_id
    '''
    if gpu_id == -1 or not torch.cuda.is_available():
        return torch.device('cpu')
    else:
        if gpu_id == None:
            p=multiprocessing.current_process()
            try:
                worker_num = int(p.name.split('-')[-1]) # Fails if it's not integer
                gpu_id = worker_num % torch.cuda.device_count()
                return torch.device(f"cuda:{gpu_id}")
            except ValueError: # Parent process
                return torch.cuda.default_stream().device




__version__ = "2021.06.09.2"
# from . import tools, data, my_random
from .tools import *
from . import my_random, os

'''
For third-party packages, import manually by something like:
import tools.pytorch
import tools.pytorch as tpytorch
import tools.pytorch as tpt

'''



import numpy as np

# %%
__all__ = [
'standardize',
'moving_mean',
]

# %%
def standardize(array, axis=None, ep=1e-20):
    return (array - array.mean(axis=axis))/(array.std(axis=axis)+ep)

def binarize(array, threshold):
    result = np.zeros_like(array)
    result[array>=threshold] = 1
    return result

def moving_mean(x, w):
    
    odd = bool(w%2)
    edge = w//2
    if odd:
        x = np.pad(x, w//2+1, mode='edge')
        x = np.cumsum(x).astype(np.float64)
        x = (x[w:] - x[:-w])/w
        return x[:-1]
    else:
        x = np.pad(x, w//2, mode='edge')
        x = np.cumsum(x).astype(np.float64)
        x = (x[w:] - x[:-w])/w
        return x

# %%
if __name__ == '__main__':
    pass
